@@ -1,40 +1,49 @@
 """Support for the Dynalite channels as covers."""
 import asyncio
+from typing import Any
 
-from .const import ATTR_POSITION, ATTR_TILT_POSITION, LOGGER
-from .dynalitebase import DynaliteMultiDevice
+from .const import ATTR_POSITION, ATTR_TILT_POSITION, CONF_TEMPLATE, CONF_TIME_COVER
+from .dynalitebase import DynaliteBaseDevice, DynaliteMultiDevice
+from .light import DynaliteChannelLightDevice
+from .switch import DynalitePresetSwitchDevice
 
 
 class DynaliteTimeCoverDevice(DynaliteMultiDevice):
     """Representation of a Dynalite Channel as a Home Assistant Cover."""
 
-    def __init__(self, area, bridge):
+    def __init__(self, area: int, bridge: Any, poll_timer: float) -> None:
         """Initialize the cover."""
-        self._current_position = 0
+        self._current_position = 0.0
         self._direction = "stop"
+        self._poll_timer = poll_timer
         super().__init__(4, area, bridge)
 
     @property
-    def category(self):
+    def available(self) -> bool:
+        """Return if device is available."""
+        return self._bridge.available(CONF_TEMPLATE, self._area, CONF_TIME_COVER)
+
+    @property
+    def category(self) -> str:
         """Return the category of the entity: light, switch, or cover."""
         return "cover"
 
     @property
-    def unique_id(self):
+    def unique_id(self) -> str:
         """Return the ID of this room switch."""
         return "dynalite_area_" + str(self._area) + "_time_cover"
 
     @property
-    def has_tilt(self):
+    def has_tilt(self) -> bool:
         """Return whether cover supports tilt."""
         return False
 
     @property
-    def device_class(self):
+    def device_class(self) -> str:
         """Return the class of the cover."""
         return self._bridge.get_device_class(self._area)
 
-    def update_level(self, actual_level, target_level):
+    def update_level(self, actual_level: float, target_level: float) -> None:
         """Update the current level."""
         if actual_level == target_level:
             self._direction = "stop"
@@ -44,152 +53,149 @@ class DynaliteTimeCoverDevice(DynaliteMultiDevice):
             self._bridge.add_timer_listener(self.timer_callback)
         self._bridge.update_device(self)
 
-    def timer_callback(self):
+    def timer_callback(self) -> None:
         """Update the progress of open and close."""
         duration = self._bridge.get_cover_duration(self._area)
-        poll_timer = self._bridge.poll_timer
+        assert self._direction in ["open", "close"]
         if self._direction == "open":
-            self._current_position += poll_timer / duration
+            self._current_position += self._poll_timer / duration
+            getattr(self, "update_tilt", int)(self._poll_timer)
             if self._current_position >= 1.0:
                 self._current_position = 1.0
                 self._direction = "stop"
                 self._bridge.remove_timer_listener(self.timer_callback)
-        elif self._direction == "close":
-            self._current_position -= poll_timer / duration
+        else:
+            assert self._direction == "close"
+            self._current_position -= self._poll_timer / duration
+            getattr(self, "update_tilt", int)(self._poll_timer)
             if self._current_position <= 0.0:
                 self._current_position = 0.0
                 self._direction = "stop"
                 self._bridge.remove_timer_listener(self.timer_callback)
-        elif self._direction == "stop":
-            self._bridge.remove_timer_listener(self.timer_callback)
-
-        if getattr(self, "update_tilt", False):
-            # pylint: disable=no-member
-            self.update_tilt(poll_timer)
-
         self._bridge.update_device(self)
 
     @property
-    def current_cover_position(self):
+    def current_cover_position(self) -> int:
         """Return the position of the cover from 0 to 100."""
         return int(self._current_position * 100)
 
     @property
-    def is_opening(self):
+    def is_opening(self) -> bool:
         """Return whether cover is currently opening."""
         return self._direction == "open"
 
     @property
-    def is_closing(self):
+    def is_closing(self) -> bool:
         """Return whether cover is currently closing."""
         return self._direction == "close"
 
     @property
-    def is_closed(self):
+    def is_closed(self) -> bool:
         """Return whether cover is closed."""
         return self._current_position == 0
 
-    async def async_open_cover(self, **kwargs):
+    async def async_open_cover(self, **kwargs) -> None:
         """Open the cover."""
+        # pylint: disable=unused-argument
         await self.get_device(1).async_turn_on()
         self.update_level(self._current_position, 1.0)
 
-    async def async_close_cover(self, **kwargs):
+    async def async_close_cover(self, **kwargs) -> None:
         """Close the cover."""
+        # pylint: disable=unused-argument
         await self.get_device(2).async_turn_on()
         self.update_level(self._current_position, 0.0)
 
-    async def async_set_cover_position(self, **kwargs):
+    async def async_set_cover_position(self, **kwargs) -> None:
         """Set the cover to a specific position."""
         target_position = kwargs[ATTR_POSITION] / 100
         position_diff = target_position - self._current_position
-        poll_timer = self._bridge.poll_timer
-        if position_diff > 0:
+        if position_diff > 0.001:
             await self.async_open_cover()
             while (
                 self._current_position < target_position and self._direction == "open"
             ):
-                await asyncio.sleep(poll_timer)
+                await asyncio.sleep(self._poll_timer)
             if self._direction == "open":
                 await self.async_stop_cover()
-                await asyncio.sleep(poll_timer)  # doing twice for safety
+                await asyncio.sleep(self._poll_timer)  # doing twice for safety
                 await self.async_stop_cover()
-        elif position_diff < 0:
+        elif position_diff < -0.001:
             await self.async_close_cover()
             while (
                 self._current_position > target_position and self._direction == "close"
             ):
-                await asyncio.sleep(poll_timer)
+                await asyncio.sleep(self._poll_timer)
             if self._direction == "close":
                 await self.async_stop_cover()
-                await asyncio.sleep(poll_timer)  # doing twice for safety
+                await asyncio.sleep(self._poll_timer)  # doing twice for safety
                 await self.async_stop_cover()
         else:
             await self.async_stop_cover()
 
-    async def async_stop_cover(self, **kwargs):
+    async def async_stop_cover(self, **kwargs) -> None:
         """Stop the cover."""
+        # pylint: disable=unused-argument
         await self.get_device(3).async_turn_on()
         self.update_level(self._current_position, self._current_position)
 
-    def listener(self, device, stop_fade):
+    def listener(self, device: DynaliteBaseDevice, stop_fade: bool) -> None:
         """Update according to updates in underlying devices."""
         if device == self.get_device(1):
-            if device.level > 0:
+            assert isinstance(device, DynalitePresetSwitchDevice)
+            if device.is_on:
                 self.update_level(self._current_position, 1.0)
         elif device == self.get_device(2):
-            if device.level > 0:
+            assert isinstance(device, DynalitePresetSwitchDevice)
+            if device.is_on:
                 self.update_level(self._current_position, 0.0)
         elif device == self.get_device(3):
-            if device.level > 0:
+            assert isinstance(device, DynalitePresetSwitchDevice)
+            if device.is_on:
                 self.update_level(self._current_position, self._current_position)
-        elif device == self.get_device(4):
+        else:
+            assert device == self.get_device(4)
+            assert isinstance(device, DynaliteChannelLightDevice)
             if stop_fade or device.direction == "stop":
                 self.update_level(self._current_position, self._current_position)
             elif device.direction == "open":
                 self.update_level(self._current_position, 1.0)
             else:
                 self.update_level(self._current_position, 0.0)
-
-        else:
-            LOGGER.error("listener received update from unknown device")
         super().listener(device, stop_fade)
 
 
 class DynaliteTimeCoverWithTiltDevice(DynaliteTimeCoverDevice):
     """Representation of a Dynalite Channel as a Home Assistant Cover that uses up and down for tilt."""
 
-    def __init__(self, area, bridge):
+    def __init__(self, area: int, bridge: Any, poll_timer: float) -> None:
         """Initialize the cover."""
         self._current_tilt = 0
-        super().__init__(area, bridge)
+        super().__init__(area, bridge, poll_timer)
 
     @property
-    def has_tilt(self):
+    def has_tilt(self) -> bool:
         """Return whether cover supports tilt."""
         return True
 
-    def update_tilt(self, poll_timer):
+    def update_tilt(self, poll_timer: float) -> None:
         """Update the current tilt based on diff and tilt_percentage."""
+        assert self._direction in ["open", "close"]
         if self._direction == "open":
             mult = poll_timer
-        elif self._direction == "close":
-            mult = -poll_timer
         else:
-            LOGGER.error(
-                "update_tilt called with invalid direction %s", self._direction
-            )
-            return
+            assert self._direction == "close"
+            mult = -poll_timer
         tilt_duration = self._bridge.get_cover_tilt_duration(self._area)
         tilt_diff = mult / tilt_duration
         self._current_tilt = max(0, min(1, self._current_tilt + tilt_diff))
 
     @property
-    def current_cover_tilt_position(self):
+    def current_cover_tilt_position(self) -> int:
         """Return the current cover tilt."""
         return int(self._current_tilt * 100)
 
-    async def apply_tilt_diff(self, tilt_diff):
+    async def apply_tilt_diff(self, tilt_diff: float) -> None:
         """Move the cover up or down based on a diff."""
         duration = self._bridge.get_cover_duration(self._area)
         tilt_duration = self._bridge.get_cover_tilt_duration(self._area)
@@ -200,23 +206,26 @@ class DynaliteTimeCoverWithTiltDevice(DynaliteTimeCoverDevice):
         )
         await self.async_set_cover_position(position=target_position)
 
-    async def async_open_cover_tilt(self, **kwargs):
+    async def async_open_cover_tilt(self, **kwargs) -> None:
         """Open the cover tilt."""
+        # pylint: disable=unused-argument
         if self._current_tilt == 1:
             return
         await self.apply_tilt_diff(1 - self._current_tilt)
 
-    async def async_close_cover_tilt(self, **kwargs):
+    async def async_close_cover_tilt(self, **kwargs) -> None:
         """Close the cover tilt."""
+        # pylint: disable=unused-argument
         if self._current_tilt == 0:
             return
         await self.apply_tilt_diff(0 - self._current_tilt)
 
-    async def async_set_cover_tilt_position(self, **kwargs):
+    async def async_set_cover_tilt_position(self, **kwargs) -> None:
         """Set the cover tilt position."""
         target_position = kwargs[ATTR_TILT_POSITION] / 100
         await self.apply_tilt_diff(target_position - self._current_tilt)
 
-    async def async_stop_cover_tilt(self, **kwargs):
+    async def async_stop_cover_tilt(self, **kwargs) -> None:
         """Stop cover tilt."""
+        # pylint: disable=unused-argument
         await self.async_stop_cover()
