@@ -1,5 +1,7 @@
 """Support for Dynalite channels as lights."""
 
+from typing import Any
+
 from .const import ATTR_BRIGHTNESS
 from .dynalitebase import DynaliteChannelBaseDevice
 
@@ -7,38 +9,33 @@ from .dynalitebase import DynaliteChannelBaseDevice
 class DynaliteChannelLightDevice(DynaliteChannelBaseDevice):
     """Representation of a Dynalite Channel as a Home Assistant Light."""
 
-    def __init__(self, area, channel, bridge):
+    def __init__(self, area: int, channel: int, bridge: Any) -> None:
         """Initialize the light."""
-        self._level = 0
+        self._level = 0.0
         self._direction = "stop"
         super().__init__(area, channel, bridge)
 
     @property
-    def category(self):
+    def category(self) -> str:
         """Return the category of the entity: light, switch, or cover."""
         return "light"
 
     @property
-    def brightness(self):
+    def brightness(self) -> int:
         """Return the brightness of this light between 0..255."""
         return int(self._level * 255)
 
     @property
-    def level(self):
-        """Return the brightness of this light between 0..1."""
-        return self._level
-
-    @property
-    def direction(self):
+    def direction(self) -> str:
         """Return the brightness of this light between 0..1."""
         return self._direction
 
     @property
-    def is_on(self):
+    def is_on(self) -> bool:
         """Return true if device is on."""
         return self._level > 0
 
-    def update_level(self, actual_level, target_level):
+    def update_level(self, actual_level: float, target_level: float) -> None:
         """Update the current level."""
         old_level = self._level
         self._level = actual_level
@@ -52,8 +49,9 @@ class DynaliteChannelLightDevice(DynaliteChannelBaseDevice):
         if self._level != old_level:
             self.update_listeners()
 
-    async def async_turn_on(self, **kwargs):
+    async def async_turn_on(self, **kwargs) -> None:
         """Turn light on."""
+        # pylint: disable=unused-argument
         if ATTR_BRIGHTNESS in kwargs:
             brightness = kwargs[ATTR_BRIGHTNESS] / 255.0
         else:
@@ -61,6 +59,7 @@ class DynaliteChannelLightDevice(DynaliteChannelBaseDevice):
         fade = self._bridge.get_channel_fade(self._area, self._channel)
         self._bridge.set_channel_level(self._area, self._channel, brightness, fade)
 
-    async def async_turn_off(self, **kwargs):
+    async def async_turn_off(self, **kwargs) -> None:
         """Turn light off."""
+        # pylint: disable=unused-argument
         await self.async_turn_on(**{ATTR_BRIGHTNESS: 0})
